-- entity_id -> { type, settings, state }

-- Need:
-- LuaLogisticNetwork: network_id, get_network_by_id
-- LuaCircuitNetwork: get_network_by_id

-- Signal Source:
--   Local Circuit Network [red/green]
--   Remote Circuit Network [index]
--   Wireless [channel]
--   Logistic Network []
--   Static Signal [signal pairs]

-- Signal to display:

-- Color Policy:
--   Default
--   Static
--   Conditioned:
--     [FROM..TO -> Color]

-- Update Rate:
--   60, 120



-- Nixie Tube
-- Nixie Tube 4x1
-- Digital Tube
-- Loading Bar
-- Button
-- Icon Plate

-- Advanced Lamp


-- AdvancedEditor => ColorPolicyEditor
-- SignalEditor
-- UpdateRateEditor


-- destruction logic (biters, self, deconstruction planner, etc)

-- Color Picker
-- Integer Editor Fixups
-- Nixie Tube Controller

--local serpent = require("serpent")
require("util")

function active_always()
  return true
end

function active_when(key, value)
  return function (self)
    return self[key] == value
  end
end

require("lib.entity")
require("lib.extension")
require("lib.message_pack.decoder")
require("lib.message_pack.encoder")
require("lib.scheduler")
require("lib.widget")

require("lib.editor")
require("lib.frame")
require("lib.gui")
require("lib.settings_box")

require("lib.color_policy")
require("lib.signal_source")
require("lib.update_rate")

------------------------------------------------------------------------------------------------

NixieTubeConfigFrame = PropertyEditorFrame:derive("tux-nixie-tube-config-frame")
NixieTubeConfigFrame.caption = "Nixie Tube Settings"
NixieTubeConfigFrame.properties = {
  { field = "update_rate", caption = "Update Rate", editor = UpdateRateEditor },
  { field = "signal_source", caption = "Signal Source", editor = SignalSourceEditor },
  { field = "signal", caption = "Signal", editor = SignalEditor }, -- !!!
  { field = "color_policy", caption = "Color Policy", editor = ColorPolicyEditor },
}

------------------------------------------------------------------------------------------------

NixieTube = Entity:derive("tux-nixie-tube-small")
NixieTube:extend(UpdateRateExt)
NixieTube:extend(SignalSourceExt)
NixieTube:extend(ColorPolicyExt)
NixieTube:extend(SettingsBoxExt)

NixieTube.config_class = NixieTubeConfigFrame
NixieTube.num_chars = 2

function NixieTube:activate(event)
  self:update_color()
end

function NixieTube:create(event)
  self:load_settings()
  -- State
  self.sprites = {}

  -- Setup
  self:create_sprites()
  self:update_color()
  -- check left and right
end

function NixieTube:create_sprites()
  local entity = self.entity
  -- local bbox = entity.bounding_box
  local step_x = (20 + 1) * 0.5 / 32
  local origin_x = 0 - (self.num_chars - 1) * (step_x / 2)

  for i = 1, self.num_chars do
    local delta_x = origin_x + (i - 1) * step_x

    local digit = rendering.draw_sprite{
      sprite = "tux-nixie-tube-sprite-30",
      surface = entity.surface,
      target = entity,
      render_layer = "object",
      target_offset = {delta_x, -4/32},
      x_scale = 0.5,
      y_scale = 0.5,
    }

    table.insert(self.sprites, digit)
  end
end

function NixieTube:on_field_changed(field_name)
  print("FIELD CHANGE " .. field_name)
  if field_name == "color" then
    self:update_color()
  end

  self:save_settings()
end

function NixieTube:on_tick()
  local x = (game.tick % 1000)
  Sheduler.add_job(self.entity, self.update_rate)
end

function NixieTube:update_color()
  local color = self.color
  local dst_color = {r = color.r / 255, g = color.g / 255, b = color.b / 255}
  for _, sprite_id in pairs(self.sprites) do
    rendering.set_color(sprite_id, dst_color)
  end
end

------------------------------------------------------------------------------------------------

function global_init()
  Entity.global_init()
  GUI.global_init()
  Scheduler.global_init()
  Widget.global_init()
end

script.on_init(global_init)
script.on_configuration_changed(global_init)

------------------------------------------------------------------------------------------------
