ColorPolicies = {}

------------------------------------------------------------------------------------------------

StaticColorConfigFrame = PropertyEditorFrame:derive("static-color-config-frame")
StaticColorConfigFrame.caption = "Static Color Settings"
StaticColorConfigFrame.properties = {
  { field = "color", caption = "Color", editor = ColorEditor },
}

function ColorPolicies.static_color(entity, _value)
  return entity.color
end

------------------------------------------------------------------------------------------------

FourColorsConfigFrame = PropertyEditorFrame:derive("four-colors-config-frame")
FourColorsConfigFrame.caption = "Color Settings"
FourColorsConfigFrame.properties = {
  { field = "color_0", caption = "Level 0 Color", editor = ColorEditor },
  { field = "color_1", caption = "Level 1 Color", editor = ColorEditor },
  { field = "color_2", caption = "Level 2 Color", editor = ColorEditor },
  { field = "color_3", caption = "Level 3 Color", editor = ColorEditor },
  { field = "value_1", caption = "Level 1 Value", editor = IntegerEditor },
  { field = "value_2", caption = "Level 2 Value", editor = IntegerEditor },
  { field = "value_3", caption = "Level 3 Value", editor = IntegerEditor },
}

function ColorPolicies.four_colors(entity, value)
  if entity.value_3 <= value then return entity.color_3 end
  if entity.value_2 <= value then return entity.color_2 end
  if entity.value_1 <= value then return entity.color_1 end
  return entity.color_0
end

------------------------------------------------------------------------------------------------

ColorPolicyEditor = ComplexVariantEditor:derive("color-policy-editor")
ColorPolicyEditor:set_variants{
  { 
    caption = "Static Color", 
    value = "static_color", 
    editor = StaticColorConfigFrame,
  },
  { 
    caption = "4 Colors with Thresholds", 
    value = "four_colors", 
    editor = FourColorsConfigFrame,
  },
}

------------------------------------------------------------------------------------------------

local active_static_color = active_when("color_policy", "static_color")
local active_four_colors = active_when("color_policy", "four_colors")

ColorPolicyExt = Extension:new()

ColorPolicyExt.properties = {
  {
    name = "color_policy",
    default = "static_color",
    caption = "Color Policy",
    editor = ColorPolicyEditor,
    active = active_always,
  },

  -- static_color properties
  {
    name = "color",
    default = {r = 255, g = 255, b = 0},
    active = active_static_color,
  },

  -- four_colors properties
  {
    name = "color_0",
    default = {r = 255, g = 0, b = 0},
    active = active_four_colors,
  },
  {
    name = "color_1",
    default = {r = 238, g = 255, b = 44},
    active = active_four_colors,
  },
  {
    name = "color_2",
    default = {r = 58, g = 255, b = 97},
    active = active_four_colors,
  },
  {
    name = "color_3",
    default = {r = 43, g = 255, b = 255},
    active = active_four_colors,
  },
  {
    name = "value_1",
    default = 100,
    active = active_four_colors,
  },
  {
    name = "value_2",
    default = 1000,
    active = active_four_colors,
  },
  {
    name = "value_3",
    default = 10000,
    active = active_four_colors,
  },
}

------------------------------------------------------------------------------------------------
