------------------------------------------------------------------------------------------------

PropertyEditorFrame = Widget:derive("property-editor-frame")
PropertyEditorFrame.caption = "Settings"
PropertyEditorFrame.properties = {}

function PropertyEditorFrame:create(parent)
  local index = #(parent.children)

  self.frame = self:add_element(parent, {
    type = "frame",
    name = string.format("settings-%i", index),
    caption = self.caption,
  })

  local rows = self.frame.add{
    type = "flow",
    name = "rows",
    direction = "vertical",
  }

  for _, prop in pairs(self.properties) do
    local row = rows.add{
      type = "flow",
      name = prop.field,
      direction = "horizontal",
    }

    local label = row.add{
      type = "label",
      name = "caption-0",
      caption = prop.caption,
      style = "tux_property_name",
    }
    
    local editor = prop.editor:new{
      ent_data = self.ent_data, 
      ent_field = prop.field,
      parent = self,
    }

    editor:create(row)
    editor:load_value(self.ent_data[prop.field])
  end

  local last_row = rows.add{
    type = "flow",
    name = "last-row",
    direction = "horizontal",
    -- style = "tux_editor_last_row",
  }

  last_row.style.horizontally_stretchable = true
  last_row.style.horizontal_align = "right"

  self:add_element(last_row, {
    type = "button",
    name = "ok",
    caption = {"gui.confirm"},
    style = "tux_editor_confirm_button"
  })
end

function PropertyEditorFrame:destroy()
  if self.valid then
    self.valid = false
    Widget.destroy_element(self.frame)
  end
end

PropertyEditorFrame:on_clicked("ok", function (self, event)
  self:destroy()
end)

------------------------------------------------------------------------------------------------



------------------------------------------------------------------------------------------------
