GUI = {}

function GUI.global_init()
  global.gui = global.gui or {}
end

------------------------------------------------------------------------------------------------

FrameStack = Widget:derive("frame-stack")

function FrameStack:clear()
  for _, child in pairs(self.stack.children) do
    Widget.destroy_element(child)
  end
end

function FrameStack:create(parent)
  self.stack = self:add_element(parent, {
    type="flow",
    name="tux-frame-stack",
  })
end

function FrameStack:from_player_id(player_id)
  local left = game.players[player_id].gui.left
  local element = left["tux-frame-stack"]

  if element then
    return Widget.from_element(element)
  else
    local frame_stack = self:new()
    frame_stack:create(left)
    return frame_stack
  end
end

function FrameStack:push_frame(index, frame)
  local children = self.stack.children
  for i = index, #children do
    Widget.destroy_element(children[i])
  end

  frame.index = index
  frame:create(self.stack)
end

function FrameStack:replace_frame(frame)
  self:clear()
  frame.index = 1
  frame:create(self.stack)
end

------------------------------------------------------------------------------------------------

script.on_event(defines.events.on_gui_closed, function (event)
  FrameStack:from_player_id(event.player_index):clear()
end)

script.on_event(defines.events.on_gui_opened, function (event)
  FrameStack:from_player_id(event.player_index):clear()
end)

------------------------------------------------------------------------------------------------
