SettingsBox = {}
SettingsBoxExt = Extension:new()

------------------------------------------------------------------------------------------------

function SettingsBox.find_settings_box(entity)
  -- try finding a ghost first
  local settings_box = entity.surface.find_entities_filtered{
    ghost_name = "tux-settings-box",
    position = entity.position
  }[1]

  if settings_box then
    _, settings_box = settings_box.revive()
    if settings_box then
      return settings_box
    end
  end

  -- try finding a non-ghost box
  local settings_box = entity.surface.find_entities_filtered{
    name = "tux-settings-box",
    position = entity.position
  }[1]

  return settings_box
end

------------------------------------------------------------------------------------------------

function SettingsBoxExt:destroy_settings_box()
  if self.settings_box then
    self.settings_box.destroy()
    self.settings_box = nil
  end
end

function SettingsBoxExt:load_settings()
  local settings_box = SettingsBox.find_settings_box(self.entity)
  if settings_box then
    self.settings_box = settings_box
  else
    return
  end

  local control_behavior = settings_box.get_control_behavior()
  if not control_behavior then
    return
  end

  local data_i32 = {}
  for i = 1, 256 do
    local signal = control_behavior.get_signal(i)
    if signal then
      table.insert(data_i32, signal.count)
    else
      break
    end
  end

  local stored_settings = MessagePack.decode_i32(data_i32)
  print("LOADED THING:")
  print(serpent.dump(stored_settings))

  for name, value in pairs(stored_settings) do
    self[name] = value
    --self:on_field_changed(name)
  end
end

function SettingsBoxExt:on_ghost_mined(ghost)
  local settings_box = SettingsBox.find_settings_box(ghost)
  if settings_box then
    settings_box.destroy()
  end
end

function SettingsBoxExt:on_mined(event)
  self:destroy_settings_box()
end

function SettingsBoxExt:save_settings()
  if not self.settings_box then
    local entity = self.entity
    local surface = entity.surface

    self.settings_box = surface.create_entity{
      name = "tux-settings-box",
      position = entity.position,
      force = entity.force,
    }
  end

  local stored_settings = {}
  for name, prop in pairs(self.properties) do
    if prop.active(self) then
      stored_settings[name] = self[name]
    end
  end

  -- print("SAVE AS")
  -- print(serpent.dump(stored_settings))

  local data_i32 = MessagePack.encode_i32(stored_settings)
  local control_behavior = self.settings_box.get_or_create_control_behavior()
  local signal = {count = 0, signal = {type = "item", name = "coal"}}
  for i, word in ipairs(data_i32) do
    signal.count = word
    control_behavior.set_signal(i, signal)
  end
end

------------------------------------------------------------------------------------------------
