widget_classes = {}

------------------------------------------------------------------------------------------------

Widget = {}

WidgetBase = {
  on_clicked_handlers = {},
  valid = true,
}

function Widget.global_init()
  global.widget_data = global.widget_data or {}
end

------------------------------------------------------------------------------------------------

function Widget:derive(name)
  return WidgetBase:derive(name)
end

function Widget.destroy_element(element)
  Widget.release_element(element)
  element.destroy()

  local count = 0
  for _ in pairs(global.widget_data) do count = count + 1 end
  print("CNT " .. count)
end

function Widget.from_data(data)
  if getmetatable(data) then
    return data
  end

  local class = widget_classes[data.type]
  if not class then
    error("warning: discarding invalid widget: " .. data.type)
  end

  setmetatable(data, class.__mt)
  return data
end

function Widget.from_element(element)
  local element_id = element.index
  local data = global.widget_data[element_id]
  if data then
    return Widget.from_data(data)
  end
end

function Widget.release_element(element)
  if not element.valid then
    return
  end

  local element_id = element.index
  local widget_data = global.widget_data[element_id]
  if widget_data then
    local widget = Widget.from_data(widget_data)
    if widget.on_destroyed then
      widget:on_destroyed()
      widget.on_destroyed = nil
    end
    global.widget_data[element_id] = nil
  end

  for _, child in pairs(element.children) do
    Widget.release_element(child)
  end
end

------------------------------------------------------------------------------------------------

function WidgetBase:add_element(parent, options)
  local element = parent.add(options)
  self:bind_element(element)
  return element
end

function WidgetBase:bind_element(element)
  global.widget_data[element.index] = self
end

function WidgetBase:derive(name)
  local class = table.deepcopy(self)
  class.__mt = {__index = class}
  class.name = name

  if not name then
    return class
  elseif not widget_classes[name] then
    widget_classes[name] = class
    return class
  else
    error("Widget with this name is already registered:" .. name)
  end
end

function WidgetBase:new(opts)
  local data = opts or {}
  data.type = self.name
  setmetatable(data, self.__mt)
  return data
end

function WidgetBase:on_changed(event)
end

function WidgetBase:on_clicked(name, func)
  self.on_clicked_handlers[name] = func
end

------------------------------------------------------------------------------------------------

function Widget.on_click(event)
  if not event.element then return end
  if not event.player_index then return end

  local widget = Widget.from_element(event.element)
  if not widget then
    return
  end

  local handler = widget.on_clicked_handlers[event.element.name]
  if handler then
    handler(widget, event)
  end
end

function Widget.on_event(event)
  if not event.element then return end
  if not event.player_index then return end

  local widget = Widget.from_element(event.element)
  if widget then
    widget:on_changed(event)
  end
end

------------------------------------------------------------------------------------------------

script.on_event(defines.events.on_gui_checked_state_changed, Widget.on_event)
script.on_event(defines.events.on_gui_click, Widget.on_click)
script.on_event(defines.events.on_gui_elem_changed, Widget.on_event)
script.on_event(defines.events.on_gui_selection_state_changed, Widget.on_event)
script.on_event(defines.events.on_gui_text_changed, Widget.on_event)
script.on_event(defines.events.on_gui_value_changed, Widget.on_event)

------------------------------------------------------------------------------------------------
