local cc = table.deepcopy(data.raw["constant-combinator"]["constant-combinator"])

local DEFAULT_INSTRUMENTS = {
  {
    name = "alarms",
    notes =
    {
      { name = "ring",  sound = { filename = "__base__/sound/programmable-speaker/ring.ogg", preload = false }},
    }
  }
}

local EMPTY_SPRITE = {
  filename = MOD.."/graphics/empty.png",
  priority = "high",
  width = 1,
  height = 1,
}

nixie_tube_small_entity =
{
  type = "programmable-speaker",
  name = "tux-nixie-tube-small",

  flags = {"not-on-map", "placeable-neutral", "player-creation"},
  icon = nixie_tube_small_item.icon,
  icon_size = nixie_tube_small_item.icon_size,
  minable = {hardness = 0.2, mining_time = 0.5, result = "tux-nixie-tube-small"},
  order = "z[zebra]",

  collision_box = {{-0.49, -0.49}, {0.49, 0.49}},
  selection_box = {{-0.5, -0.5}, {0.5, 0.5}},

  -- entity-with-health
  corpse = "small-remnants",
  max_health = 200,
  resistances =
  {
    {
      type = "fire",
      percent = 100
    },
    {
      type = "physical",
      percent = 50
    },
  },

  -- programmable-speaker
  circuit_connector_sprites = nil, -- !!!!
  circuit_wire_connection_point =
  {
    shadow = {
      red = {12/32, -23/32},
      green = {12/32, -20/32},
    },
    wire = {
      red = {12/32, -25/32},
      green = {12/32, -21/32},
    }
  },
  circuit_wire_max_distance = 7.5,
  energy_source =
  {
    type = "electric",
    usage_priority = "secondary-input",
  },
  energy_usage_per_tick = "4KW",
  instruments = DEFAULT_INSTRUMENTS,
  maximum_polyphony = 1,
  sprite =
  {
    filename = MOD.."/graphics/nixie-tube-small-base.png",
    priority = "high",
    width = 48,
    height = 42,
    frame_count = 1,
    axially_symmetrical = false,
    direction_count = 1,
    shift = {8/32, -6/32}
  },
}

settings_box_entity =
{
  type = "programmable-speaker",
  name = "tux-settings-box",

  flags = {
    "hide-alt-info",
    "not-flammable",
    "not-on-map", 
    "not-rotatable",
    "not-selectable-in-game",
    "player-creation",
  },

  icon = settings_box_item.icon,
  icon_size = settings_box_item.icon_size,
  minable = {hardness = 0.2, mining_time = 0.5, result = "tux-nixie-tube-small"},
  order = "z[zebra]",

  collision_box = {{-0.5, -0.5}, {0.5, 0.5}},
  collision_mask = {"layer-15"},
  selection_box = nil,

  tile_width = 1,
  tile_height = 1,

  -- entity-with-health
  corpse = "small-remnants",
  max_health = 200,
  resistances =
  {
    {
      type = "fire",
      percent = 100
    },
    {
      type = "physical",
      percent = 50
    },
  },

  -- programmable-speaker
  circuit_connector_sprites = nil,
  circuit_wire_max_distance = 7.5,
  energy_source = { type = "void" },
  energy_usage_per_tick = "1KW",
  instruments = DEFAULT_INSTRUMENTS,
  maximum_polyphony = 1,
  sprite =
  {
    filename = MOD.."/graphics/empty.png",
    priority = "high",
    width = 1,
    height = 1,
    frame_count = 1,
    axially_symmetrical = false,
    direction_count = 1,
  },
}

settings_box_entity_2 =
{
  type = "constant-combinator",
  name = "tux-settings-box",

  flags = {
    "hide-alt-info",
    "not-deconstructable",
    "not-flammable",
    "not-on-map", 
    "not-rotatable",
    --"not-selectable-in-game",
    "player-creation",
  },

  icon = settings_box_item.icon,
  icon_size = settings_box_item.icon_size,
  minable = nil, --{hardness = 0.2, mining_time = 0.5, result = "tux-nixie-tube-small"},
  order = "z[zebra]",

  collision_box = {{-0.49, -0.49}, {0.49, 0.49}},
  collision_mask = {"layer-15"},
  selection_box = {{-0.25, -0.25}, {0.25, 0.25}}, -- nil

  tile_width = 1,
  tile_height = 1,

  -- entity-with-health
  corpse = "small-remnants",
  max_health = 200,
  resistances =
  {
    {
      type = "fire",
      percent = 100
    },
    {
      type = "physical",
      percent = 50
    },
  },

  -- constant-combinator
  activity_led_sprites = cc.activity_led_sprites,
  activity_led_light = { intensity = 0, size = 0 },
  activity_led_light_offsets = cc.activity_led_light_offsets,
  circuit_wire_connection_points = cc.circuit_wire_connection_points,
  item_slot_count = 256,
  sprites = {
    north = EMPTY_SPRITE,
    south = EMPTY_SPRITE,
    west = EMPTY_SPRITE,
    east = EMPTY_SPRITE,
  },
}

data:extend({
  nixie_tube_small_entity,
  settings_box_entity_2,
})
